package cc.smtweb.system.bpm.web.engine.model.listcard.mastersub;

import cc.smtweb.framework.core.common.R;
import cc.smtweb.framework.core.common.SwMap;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.EntityDao;
import cc.smtweb.framework.core.db.cache.ModelTableCache;
import cc.smtweb.framework.core.db.impl.DefaultEntity;
import cc.smtweb.framework.core.db.jdbc.IDbWorker;
import cc.smtweb.framework.core.db.vo.ModelTable;
import cc.smtweb.framework.core.exception.BizException;
import cc.smtweb.system.bpm.web.design.form.define.PageDataset;
import cc.smtweb.system.bpm.web.engine.model.listcard.ModelUtils;
import cc.smtweb.system.bpm.web.engine.model.listcard.abshandler.AbsSaveHandler;
import cc.smtweb.system.bpm.web.engine.model.listcard.single.LCSingleHelper;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.Set;

/**
 * @Author:lip
 * @Date : 2022/8/28 21:03
 */
public class LCMsSaveHandler extends AbsSaveHandler<DefaultEntity> {
    /**
     * 保存指定数据集操作，入参：{pageId,dataset:"", data:{form:{},list: {total:0,rows:[]}}, filter:{}}
     */
    public R save() {
        //对应的数据集定义
        PageDataset masterDataSet = ModelUtils.findPageDatasetByName(this.datasets, LCMsDefine.dsMCard);
        PageDataset subDataSet = ModelUtils.findPageDatasetByName(this.datasets, LCMsDefine.dsSList);
        //
        SwMap filter = params.readMap("filter");

        //待保存数据
        SwMap data = params.readMap("data");
        if (data == null) throw new BizException("没有收到待保存的的数据：" + LCMsDefine.dsMCard + "！");

        //获取子表关联字段
        String subForeignKey = form.getOpts().readMap("config").readMap("props").readString("subForeignKey", "-1");
        Assert.isTrue(!subForeignKey.equals("-1"), "未获取到子表外键配置");

        //读取待保存的bean
        DefaultEntity masterBean = readBeanFromPage(masterDataSet, data.readMap(LCMsDefine.dsMCard).readMap("form"));
        if (filter != null && masterBean.isNew()) {
            //有过滤条件，将关联的值设上
            setLinkValue(masterDataSet, masterBean, f -> filter.get(f.name));
        }
        // 读取子表数据
        SwMap details = params.readMap("details");
        Assert.notNull(details, "没有收到待保存的明细数据");
        //
        ArrayList<DefaultEntity> inserted = new ArrayList<>();
        ArrayList<DefaultEntity> updated = new ArrayList<>();
        details.readListMap("inserted").forEach(row -> {
            SwMap rowData = new SwMap();
            rowData.putAll(row);
            DefaultEntity insertBean = readBeanFromPage(subDataSet, rowData);
            insertBean.put(subForeignKey, masterBean.getEntityId());
            inserted.add(insertBean);
        });
        details.readListMap("updated").forEach(row -> {
            SwMap rowData = new SwMap();
            rowData.putAll(row);
            DefaultEntity updateBean = readBeanFromPage(subDataSet, rowData);
            updated.add(updateBean);
        });
        Set<Long> deleted = details.readLongSet("deleted");

        //
        checkBean(masterBean);

        DbEngine.getInstance().doTrans(new IDbWorker() {
            @Override
            public void work() {
                saveBean(masterBean);
                ModelTable subTable = ModelTableCache.getInstance().get(subDataSet.masterTable);
                Assert.notNull(subTable, "未获取到子表");

                EntityDao<DefaultEntity> dao = DbEngine.getInstance().findDao(subTable.getName());
                dao.batchInsertEntity(inserted);
                dao.batchUpdateEntity(updated);
                if (deleted != null) {
                    dao.deleteEntity(deleted);
                }
            }

            @Override
            public void doAfterDbCommit() {
                afterCommit(masterBean);
            }

            @Override
            public void doAfterDbRollback() {
                afterRollback(masterBean);
            }
        });
        return R.success(LCSingleHelper.buildRetData(masterDataSet.name, masterBean.getData()));
    }
}
